/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id: SetRequiredBundleVersionRangesTest.java,v 1.2 2006/06/20 22:37:59 marcelop Exp $
 */
package org.eclipse.emf.build.ant.tests;

import java.util.Arrays;

import org.eclipse.emf.build.ant.taskdefs.SetRequiredBundleVersionRanges;
import org.eclipse.emf.build.ant.taskdefs.SetRequiredBundleVersionRanges.VersionFormula;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * @since 2.2.0
 */
public class SetRequiredBundleVersionRangesTest extends TestCase
{
  public static class MySetRequiredBundleVersionRanges extends SetRequiredBundleVersionRanges
  {
    public String[] splitRequiredBundles(String requiredBundles)
    {
      return super.splitRequiredBundles(requiredBundles);
    }

    public String setRequiredBundleVersionRange(String manifestContent, String symbolicName, String versionRange)
    {
      return super.setRequiredBundleVersionRange(manifestContent, symbolicName, versionRange);
    }
  }

  protected MySetRequiredBundleVersionRanges task;

  public SetRequiredBundleVersionRangesTest(String name)
  {
    super(name);
  }

  public static Test suite()
  {
    TestSuite testSuite = new TestSuite("SetRequiredBundleVersionRangesTest");
    testSuite.addTest(new SetRequiredBundleVersionRangesTest("testComputeVersion"));
    testSuite.addTest(new SetRequiredBundleVersionRangesTest("testSplitRequiredBundles"));
    testSuite.addTest(new SetRequiredBundleVersionRangesTest("testSetRequiredBundleVersionRange"));
    return testSuite;
  }

  protected void setUp() throws Exception
  {
    task = new MySetRequiredBundleVersionRanges();
  }

  public void testComputeVersion() throws Exception
  {
    assertEquals("3.0.0.qualifier1", new VersionFormula("3.0.0.qualifier1").toVersion("2.2.0.qualifier"));
    assertEquals("3.0.0", new VersionFormula("3.0.0").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0.qualifier1", new VersionFormula("3.{current}.0.qualifier1").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0.qualifier", new VersionFormula("3.{current}.0.{current}").toVersion("2.2.0.qualifier"));
    assertEquals("3.3.0.qualifier", new VersionFormula("3.{current+1}.0.{current}").toVersion("2.2.0.qualifier"));
    assertEquals("3.1.0.qualifier", new VersionFormula("3.{current-1}.{current-1}.{current}").toVersion("2.2.0.qualifier"));

    assertEquals("3.0.0.qualifier1", new VersionFormula("3.0.0.qualifier1").toVersion("2.2"));
    assertEquals("3.0.0", new VersionFormula("3.0.0").toVersion("2"));
    assertEquals("3.2.0.qualifier1", new VersionFormula("3.{current}.0.qualifier1").toVersion("2.2"));
    assertEquals("3.2.0", new VersionFormula("3.{current}.0.{current}").toVersion("2.2.0"));
    assertEquals("3.3.0", new VersionFormula("3.{current+1}.0.{current}").toVersion("2.2"));
    assertEquals("3.1.0", new VersionFormula("3.{current-1}.{current-1}.{current}").toVersion("2.2"));
    assertEquals("3.0.0", new VersionFormula("3.{current-1}.{current-1}.{current}").toVersion("2"));

    assertEquals("3.0", new VersionFormula("3.0").toVersion("2.2.0.qualifier"));
    assertEquals("3", new VersionFormula("3").toVersion("2.2.0.qualifier"));
    assertEquals("3.2", new VersionFormula("3.{current}").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0", new VersionFormula("3.{current}.0").toVersion("2.2.0.qualifier"));
    assertEquals("3.3", new VersionFormula("3.{current+1}").toVersion("2.2.0.qualifier"));
    assertEquals("3.1", new VersionFormula("3.{current-1}").toVersion("2.2.0.qualifier"));

    assertEquals("2.2.0.qualifier", new VersionFormula("{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("2.2.0", new VersionFormula("{current}.{current}.{current}").toVersion("2.2.0.qualifier"));

    assertEquals("3.0.0.qualifier", new VersionFormula("3.0.{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0.qualifier", new VersionFormula("3.{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0.qualifier", new VersionFormula("3.{current}.{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("3.2.0.qualifier", new VersionFormula("3.{current}.0.{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("3.3.0.qualifier", new VersionFormula("3.{current+1}.{current*}").toVersion("2.2.0.qualifier"));
    assertEquals("3.1.0.qualifier", new VersionFormula("3.{current-1}.{current*}").toVersion("2.2.0.qualifier"));
  }

  public void testSplitRequiredBundles() throws Exception
  {
    String[] array = new String []
    {
      "org01",
      "org02",
      "org03;bundle-version=\"[2.0, 3.0]\"",
      "org04;bundle-version=\"[2.0, 3.0]\"",
      "org05;bundle-version=\"(2.0, 3.0]\"",
      "org06;bundle-version=\"(2.0, 3.0]\"",
      "org07;bundle-version=\"[2.0, 3.0)\"",
      "org08;bundle-version=\"[2.0, 3.0)\"",
      "org09;bundle-version=\"(2.0, 3.0)\"",
      "org10;bundle-version=\"(2.0, 3.0)\"", 
      "org11   ; \n\r\n bundle-version\n\n=      \"(2.0, 3.0)\""
    };

    String[] expectedArray = new String []
    {
      "org01",
      "org02",
      "org03[2.0,3.0]",
      "org04[2.0,3.0]",
      "org05(2.0,3.0]",
      "org06(2.0,3.0]",
      "org07[2.0,3.0)",
      "org08[2.0,3.0)",
      "org09(2.0,3.0)",
      "org10(2.0,3.0)",
      "org11(2.0,3.0)"
    };

    StringBuffer requiredBundles = new StringBuffer();
    for (int i = 0; i < array.length; i++)
    {
      requiredBundles.append(",").append(array[i]);
      if (i % 2 == 0)
      {
        requiredBundles.append(";visibility:=reexport");
      }
    }
    requiredBundles.deleteCharAt(0);

    String[] requiredBundleArray = task.splitRequiredBundles(requiredBundles.toString());
    assertTrue("Arrays are different", Arrays.equals(expectedArray, requiredBundleArray));
  }

  public void testSetRequiredBundleVersionRange() throws Exception
  {
    String nl = System.getProperty("line.separator");

    StringBuffer content = new StringBuffer()
      .append("...")
      .append(nl).append("...")
      .append(nl).append("Require-Bundle: org1,")
      .append(nl).append(" org2;bundle-version=\"[2.0,3.0)\",")
      .append(nl).append(" org3;bundle-version=\"(2.0, 3.0)\";visibility:=reexport,")
      .append(nl).append(" org4;visibility:=reexport;bundle-version=\"[2.0, 3.0]\",")
      .append(nl).append(" org5;visibility:=reexport;bundle-version=\"[2.0,3.0)\",")
      .append(nl).append(" org61 ; resolution:=optional ; bundle-version = \"(2.2.0, 3.0.0)\",")
      .append(nl).append(" org6 ; resolution:=optional ; bundle-version = \"(2.2.0, 3.0.0)\",")
      .append(nl).append(" org6.1 ; resolution:=optional ; bundle-version = \"(2.2.0, 3.0.0)\",")
      .append(nl).append(" org7 ; ")
      .append(nl).append("  resolution:=optional ;")
      .append(nl).append("      bundle-version = \"[2.2.0, 3.0.0]\"")
      .append(nl);

    StringBuffer expected = new StringBuffer()
      .append("...")
      .append(nl).append("...")
      .append(nl).append("Require-Bundle: org1;bundle-version=\"[init, end)\",")
      .append(nl).append(" org2;bundle-version=\"[init, end)\",")
      .append(nl).append(" org3;visibility:=reexport;bundle-version=\"[init, end)\",")
      .append(nl).append(" org4;visibility:=reexport;bundle-version=\"[init, end)\",")
      .append(nl).append(" org5;visibility:=reexport;bundle-version=\"[init, end)\",")
      .append(nl).append(" org61 ; resolution:=optional ;bundle-version=\"[init61, end61)\",")
      .append(nl).append(" org6 ; resolution:=optional ;bundle-version=\"[init, end)\",")
      .append(nl).append(" org6.1 ; resolution:=optional ;bundle-version=\"[init6.1, end6.1)\",")
      .append(nl).append(" org7 ; ")
      .append(nl).append("  resolution:=optional ;bundle-version=\"[init, end)\"")
      .append(nl);

    String modifiedContent = content.toString();
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org1", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org2", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org3", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org4", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org5", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org61", "[init61, end61)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org6", "[init, end)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org6.1", "[init6.1, end6.1)");
    modifiedContent = task.setRequiredBundleVersionRange(modifiedContent, "org7", "[init, end)");

    assertEquals(expected.toString(), modifiedContent);
  }
}
