/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id: BaseTask.java,v 1.1 2006/06/19 17:29:15 marcelop Exp $
 */
package org.eclipse.emf.build.ant.taskdefs;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;


/**
 * Base class for the tasks that are defined in this plugin.  Provides common behavior 
 * and facilities.
 * 
 * @since 2.2.0
 */
public abstract class BaseTask extends Task
{
  /**
   * Throws a <tt>BuildException</tt> if <tt>expression</tt> is false. 
   * @param message
   * @param expression
   * @throws BuildException
   */
  public static void assertTrue(String message, boolean expression) throws BuildException
  {
    if (!expression)
    {
      throw new BuildException(message);
    }
  }
  
  private int verbosity = Project.MSG_VERBOSE;

  public void setVerbose(boolean verbose)
  {
    this.verbosity = verbose ? Project.MSG_INFO : Project.MSG_VERBOSE;
  }
  
  public void log(String msg)
  {
    log(msg, verbosity);
  }

  
  public final void execute() throws BuildException
  {
    checkAttributes();

    try
    {
      doExecute();
    }
    catch (Exception e)
    {
      if (e instanceof BuildException)
      {
        throw (BuildException)e;
      }
      else
      {
        throw new BuildException(e);
      }
    }
  }

  /**
   * All the attribute checks should be performed in this method. 
   * @throws BuildException
   */
  protected void checkAttributes() throws BuildException
  {
  }

  /**
   * Performs the task specific code.
   * @throws Exception
   */
  abstract protected void doExecute() throws Exception;
}